#ifndef LTM_TEMPLATE_INTERNAL
#define LTM_TEMPLATE_INTERNAL

#include "ltm/template.h"

typedef enum ltm_placeholder_type {
  ltm_placeholder_type_invalid = 0,
  ltm_placeholder_type_var,
  ltm_placeholder_type_loop_start,
  ltm_placeholder_type_loop_end,
} ltm_placeholder_type;

/**
 * Internal struct used when compiling templates
 */
typedef struct ltm_placeholder {
  const char *start;
  const char *end;
  struct {
    const char *s;
    size_t len;
  } name;
  ltm_placeholder_type type;
} ltm_placeholder;

/**
 * The type of a template block
 */
typedef enum ltm_template_block_type {
  ltm_template_block_type_literal = 0,
  ltm_template_block_type_var,
  ltm_template_block_type_loop,
} ltm_template_block_type;

/**
 * A block in a template.
 */
typedef struct ltm_template_block {
  ltm_template_block_type type;
  union {
    ltm_template *template;
    const char *s;
  } data;
  size_t len;
} ltm_template_block;

typedef struct ltm_template_block_name {
  struct {
    const char *s;
    size_t len;
  } name;
  size_t index;
} ltm_template_block_name;

/**
 * Represents a compiled template. A template consists of a list of blocks, of
 * which some have names that can be indexed (e.g. variables, loops).
 */
struct ltm_template {
  struct {
    ltm_template_block *arr;
    size_t len;
  } blocks;
  struct {
    ltm_template_block_name *arr;
    size_t len;
  } names;
};

ltm_err ltm_template_init(ltm_template **out);

#endif
